import femm
#pip install pyfemm

#Variables
UNITS = "millimeters"
MODEL_THICKNESS = 100

MEDIUM = "Air"

MAG_COUNT = 4
MAG_HEIGHT = 5
MAG_WIDTH = 5
MAG_MATERIAL = "BN10"
MAG_DIRECTION = 90
MAG_INSERT = MAG_COUNT - 1

SHEET_HEIGHT = 1
SHEET_WIDTH= 19
SHEET_MATERIAL = "1010 Steel"

C_STEEL_OFFSET = 20
C_STEEL_THICKNESS = 5
C_STEEL_MATERIAL = "1010 Steel"

START_X = 0
START_Y = 0

def draw_point_array(points):
    for point in points:
        femm.mi_addnode(point[0], point[1])

    for i in range(0, len(points)-1):

        femm.mi_addsegment(points[i][0], points[i][1], points[i+1][0], points[i+1][1])
    femm.mi_addsegment(points[len(points)-1][0], points[len(points)-1][1], points[0][0], points[0][1])
def create_magnets(mag_count, mag_height, mag_width, mag_material, mag_direction, sheet_thickness, c_steel_thickness, start_x = 0, start_y = 0, sheet_insert = None):
    #Hacky way to define default place to insert sheet
    if sheet_insert == None:
        sheet_insert = mag_count - 2
    sheet_flag = False
    cur_y = start_y + c_steel_thickness
    cur_x = start_x

    #define magnets
    for i in range(0, mag_count):
        #If desired height for sheet insert has been detected then factor into magnet spacing
        if i > sheet_insert and sheet_flag == False:
            cur_y += sheet_thickness
            sheet_flag = True

        points=[[cur_x, cur_y],
        [cur_x, cur_y + mag_height],
        [cur_x + mag_width, cur_y + mag_height],
        [cur_x + mag_width, cur_y]]

        draw_point_array(points)

        femm.mi_addblocklabel((2*cur_x+mag_width)/2, (2*cur_y+mag_height)/2)
        femm.mi_selectlabel((2*cur_x+mag_width)/2, (2*cur_y+mag_height)/2)
        femm.mi_setblockprop(mag_material, 0, 0, 0, mag_direction)

        cur_y += mag_height

def create_sheet(thickness, width, material, start_x=0, start_y=0):
    points=[[start_x, start_y],
            [start_x, start_y + thickness],
            [start_x + width, start_y + thickness],
            [start_x + width, start_y]]
    draw_point_array(points)

    femm.mi_addblocklabel((2 * start_x + width) / 2, (2 * start_y + thickness) / 2)
    femm.mi_selectlabel((2 * start_x + width) / 2, (2 * start_y + thickness) / 2)
    femm.mi_setblockprop(material)

    #Create group so easier to iterate through movements of the sheet
    for point in points:
        femm.mi_selectnode(point[0], point[1])
        femm.mi_setnodeprop('', groupno="1")

    # Hacky, but works
    femm.mi_selectgroup(1)
    femm.mi_movetranslate(-width + MAG_WIDTH,0)



#Create shorting loop
def create_loop(thickness, offset, material, mag_height, mag_count, sheet_thickness, start_x = 0, start_y = 0):
    #Draw nodes
    points = [[start_x, start_y],
    [start_x + offset + thickness, start_y],
    [start_x + offset + thickness, start_y + 2 * thickness + mag_height * mag_count + sheet_thickness],
    [start_x, start_y + 2 * thickness + mag_height * mag_count + sheet_thickness],
    [start_x, start_y + thickness + mag_height * mag_count + sheet_thickness],
    [start_x + offset, start_y + thickness + mag_height * mag_count + sheet_thickness],
    [start_x + offset, start_y + thickness],
    [start_x, start_y + thickness]]

    draw_point_array(points)
    #Add and define block label
    femm.mi_addblocklabel(start_x+offset+(thickness/2), (start_y+start_y + 2 * thickness + mag_height * mag_count + sheet_thickness)/2)
    femm.mi_selectlabel(start_x+offset+(thickness/2), (start_y+start_y + 2 * thickness + mag_height * mag_count + sheet_thickness)/2)
    femm.mi_setblockprop(material)


femm.openfemm()
femm.newdocument('m')
femm.mi_probdef(0, 'millimeters', 'planar', 1e-008, MODEL_THICKNESS)

femm.mi_getmaterial(C_STEEL_MATERIAL)
femm.mi_getmaterial(MAG_MATERIAL)
femm.mi_getmaterial(SHEET_MATERIAL)
femm.mi_getmaterial(MEDIUM)

create_loop(C_STEEL_THICKNESS, C_STEEL_OFFSET, C_STEEL_MATERIAL, MAG_HEIGHT, MAG_COUNT, SHEET_HEIGHT,  START_X, START_Y)
create_magnets(MAG_COUNT, MAG_HEIGHT, MAG_WIDTH, MAG_MATERIAL, MAG_DIRECTION, SHEET_HEIGHT, C_STEEL_THICKNESS, START_Y, START_Y)

#the START_X-0.01 is so the sheet nodes don't line up with the magnet nodes and make it impossible to iterate through movement if that is desired
create_sheet(SHEET_HEIGHT, SHEET_WIDTH, SHEET_MATERIAL, START_X-0.01, START_Y + MAG_INSERT*MAG_HEIGHT + C_STEEL_THICKNESS)

#DEFINE MEDIUM
femm.mi_addblocklabel(START_X-1,START_Y-1)
femm.mi_selectlabel(START_X-1,START_Y-1)
femm.mi_setblockprop(MEDIUM)

femm.mi_addblocklabel(START_X+C_STEEL_OFFSET/2, C_STEEL_THICKNESS + 1)
femm.mi_selectlabel(START_X+C_STEEL_OFFSET/2, C_STEEL_THICKNESS + 1)
femm.mi_setblockprop(MEDIUM)



femm.mi_makeABC()
femm.mi_saveas("test.fem")
femm.mi_createmesh()
femm.mi_analyse(0)